import pytest
from src.dreamtools import file_manager

from src.dreamtools.controller_manager import ControllerEngine
from src.dreamtools.mailing_manager import MailController
from src.dreamtools.tracking_manager import TrackingManager


class Constantine(ControllerEngine):
    mailer:MailController

# 🔧 Fixture d'initialisation principale
@pytest.fixture(scope="session")
def fixation():

    application_name = "app_name"
    application_directory = file_manager.execution_directory()

    ControllerEngine.initialize(application_name, application_directory)

    path_log = file_manager.path_build(Constantine.APP_DIR, 'konfigurator/log.yml')
    TrackingManager.initialisation(path_log, logger='development', project_name=application_name)

    # Si tu veux faire un nettoyage après





# -*- coding: utf-8 -*-
# mail_objecting_manager.py

__all__ = ['MailController']

import os
from email import encoders
from email.mime.base import MIMEBase

"""
Module de Gestion de mail_object préparés

pathfile : dreamtools-dreamgeeker/mail_objecting_manager.py

Pré-Requis
-------------
.. warning::

    Indiquer les parametres smtp dans le fichiers de configuration <PROJECT_NAME>/cfg/.app.yml

.. code-block:: YAML

    SMTP_HOST: smtp-host_adresse
    SMTP_PORT: port_smtp
    SMTP_AUTHmail_object: mail_object_authen
    SMTP_AUTHPWD: password_auth
    SMTP_USERNAME : name_sender <template_email>

.. warning::

    Les mail_objects sont à définir dans le ficchier <PROJECT_NAME>/cfg/mail_objecting.yml au format suivant

.. code-block:: YAML

     footer:
      html: <Pied de mail_object unique pour tous les mail_objects (signature, rgpd...)>
      text: <Pied de mail_object unique pour tous les mail_objects (signature, rgpd...)>
     code_mail_object:
      html: <ici mail_object au format HTML>
      text : <Le mail_object au format texte>
      objt : <Objet du mail_object>


"""
import dataclasses
import smtplib
import ssl

from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText

from .config_manager import ConfigController
from .tracking_manager import TrackingManager


@dataclasses.dataclass
class MailController:
    SMTP_HOST: str
    SMTP_PORT: int
    SMTP_AUTH_EMAIL: str
    SMTP_AUTH_PWD: str
    PATH_TEMPLATES: str # = '.cfg/mail_objecting.yml'
    SMTP_USER_NAME:str = 'ne-pas-repondre'
    SMTP_IDENTITY: str =''

    def __post_init__(self):
        self.SMTP_IDENTITY = f"{self.SMTP_USER_NAME} <{self.SMTP_AUTH_EMAIL}>"

    def send_mail(self, subject:str, receivers:str, d_msg, to_receiver:str =None, **kwargs):
        """ Envoie du mail_object

        :param subject: Sujet du mail_object
        :param receivers: template_email destinataire
        :param d_msg: Message
        :param to_receiver: Nom destinataire
        :return:
        """

        TrackingManager.flag("SEND_mail_object : Paramétrage smtp")
        context = ssl.create_default_context()

        TrackingManager.flag("SEND_mail_object:Paramétrage message MIME")
        message = MIMEMultipart("alternative")
        message["Subject"] = subject
        message["From"] = self.SMTP_IDENTITY
        message["To"] = to_receiver or receivers

        if 'cc' in kwargs:
            message["Cc"] = kwargs.pop('cc')

        if 'cci' in kwargs:
            message["Cci"] = kwargs.pop('cci')

        TrackingManager.flag("SEND_mail_object:Paramétrage contenu mail_object")
        content = d_msg.get('text')
        content = MIMEText(content, 'plain', 'utf-8')
        message.attach(content)

        if d_msg.get('html'):
            content = d_msg.get('html')
            content = MIMEText(content, "html")
            message.attach(content)

        if kwargs.get('file'):
            attachements = kwargs.pop('file')
            filename = os.path.basename(attachements)

            with open(attachements, "rb") as attachment:
                content = MIMEBase('wwwcwest', 'octate-stream', Name=filename)
                content.set_payload(attachment.read())

            encoders.encode_base64(content)
            content.add_header('Content-Disposition', f"attachment'; filename={filename}")
            message.attach(content)

        TrackingManager.flag("SEND_mail_object:Connexion SMTP")
        with smtplib.SMTP_SSL(self.SMTP_HOST, self.SMTP_PORT, context=context) as server:
            TrackingManager.flag("SEND_mail_object:Authentification")
            server.login(self.SMTP_AUTH_EMAIL, self.SMTP_AUTH_PWD)
            TrackingManager.flag("SEND_mail_object: Sending")
            server.sendmail(self.SMTP_AUTH_EMAIL, receivers, message.as_string())

        return True

    def load_mail(self, code:str):
        mailer = ConfigController.json_loading(self.PATH_TEMPLATES, 'data')
        template_email = mailer[code]
        footer = mailer.get('footer') or {'text': '', 'html': ''}

        template_email['text'] += footer['text']
        template_email['html'] += footer['html']

        del mailer

        return template_email

    async def presend(self, email:str, code:str, dest_name:str = '', attachement=None, dest_cc=None, dest_cci=None, **data_field):
        """
            Prépare un e-mail avant envoi en chargeant un modèle de message et en injectant les données.

            Cette fonction charge un template d'e-mail (texte + HTML + sujet) selon un code donné,
            le personnalise avec les données fournies, puis appelle la fonction d'envoi.

            Parameters
            ----------
            email : str
                Adresse e-mail du destinataire.
            code : str
                Clé de référence du modèle à charger. Si 'custom', alors un template personnalisé doit être passé via `template_data`.
            dest_name : str, optional
                Nom du destinataire (ex. "Mme Dupont"), utilisé pour l'affichage ou l'injection dans le template.
            **data_field :
                Données de remplissage du template (ex. nom, date, référence). Si `code == 'custom'`, un champ `template_data` est attendu :
                `template_data = {'text': str, 'html': str, 'subject': str}`.

            Returns
            -------
            bool
                `True` si l'envoi s'est bien déroulé, sinon `False`.

            Example
            -------
            presend (
                email="client@example.com",
                code="confirmation",
                dest_name="Mme Dupont",
                nom="Marie",
                date="16 juillet 2025"
            )
            :param dest_name:
            :param code:
            :param email:
            :param dest_cci:
            :param dest_cc:
            :param attachement: """
        TrackingManager.flag(f'PRESEND:Loading template {code}')
        if code == 'custom' and data_field.get('template_email'):
            template_email = data_field['template_email']
        else:
            template_email = self.load_mail(code)

        TrackingManager.flag('PRESEND: Preparation')

        data_field['dest_name']= dest_name

        part1 = template_email['text'].format(**data_field)
        part2 = template_email['html'].format(**data_field)

        to_receiver = fr'{dest_name} <{email}>'

        item_tracking =f'**************************** Envoi ({code}) -> {email}'
        TrackingManager.flag(item_tracking)
        send = TrackingManager.fntracker(self.send_mail, item_tracking, template_email.get('subject'),
                                         email, {'text': part1, 'html': part2}, to_receiver,
                                         attachment=attachement, cc=dest_cc, cci=dest_cci)

        return send



_all_ = ['ControllerEngine']
from . import file_manager


class ControllerEngine:
    APP_NAME:str
    PROJECT_DIR:str
    APP_DIR:str
    TMP_DIR:str

    @classmethod
    def initialize(cls, project_name:str, application_path:str):
        cls.APP_NAME = project_name
        cls.APP_DIR = application_path
        cls.PROJECT_DIR = file_manager.parent_directory(application_path)
        cls.TMP_DIR = file_manager.path_build(cls.APP_DIR, '.tmp')
        file_manager.makedirs(cls.TMP_DIR)




class Reponce:
    status:int
    detail = None
    message:str
    title:str

    def __init__(self, message:str='Opération réussie', title:str='', status:int=200, *args, **kwargs):
        self.status = status
        self.detail = args if len(args) > 1 else args[0] if len(args)== 1 else kwargs
        self.message = message
        self.title = title


    @property
    def is_success(self):
        """ Renvoie le status de la réponse

        :rtype: bool
        """
        return 200 <= self.status < 300



class ExceptionManager(Exception):
    """
    Exception personnalisée avec suivi automatique via le tracker.

    :param message: Message lisible destiné à l’utilisateur.
    :param status: Code HTTP de l’erreur.
    :param title: Titre bref, destiné aux logs et à l’interface.
    """

    def __init__(self, message="Une erreur interne est survenue.", title='ERROR SYSTEM', status=500, **kwargs):
        super().__init__(message, title, status)

        self.message = message
        self.title = title
        self.status = status
        self.detail = kwargs


    def __str__(self):
        return str(f'[{self.title}] : {self.message} - {self.status}')

    @property
    def is_success(self):
        return 200 <= self.status < 300

class RequestException(ExceptionManager):
    def __init__(self):
        super().__init__(
            "Le format ou le contenu de la requête n’est pas accepté.",
            "Requête non acceptable",
            406
        )


class AuthException(ExceptionManager):
    def __init__(self , message="Identifiants incorrects. Veuillez vérifier votre login ou mot de passe."):
        super().__init__(
            message,
            "Échec d’authentification",
            401
        )


class OAuthException(ExceptionManager):
    def __init__(self,
                 message="Vous n’avez pas les autorisations nécessaires pour accéder à cette ressource.",
                 title="Accès interdit"):
        super().__init__(message,title, 403)


class OAuthElapsedException(ExceptionManager):
    def __init__(self,
                 message="Le délai d’authentification a expiré. Veuillez recommencer",
                 title="Délai dépassé"):
        super().__init__(message, title, 408)


class UExistException(ExceptionManager):
    def __init__(self, message="Ce compte existe déjà. Veuillez utiliser une autre adresse ou réinitialiser votre mot de passe.",
                 title="Conflit d'identité"):
        super().__init__(message, title, 409)


class AccountException(ExceptionManager):
    def __init__(self,
                 message="Aucun compte correspondant n’a été trouvé.",
                 title="Compte introuvable"):
        super().__init__(message, title, 404)

class RessourceException(ExceptionManager):
    def __init__(self,
                 message="La ressource demandée est introuvable ou n’existe plus.",
                 title="Ressource introuvable"):
        super().__init__(message, title, 404)

class PageException(ExceptionManager):
    def __init__(self, message="La page que vous cherchez n’existe pas.",
                 title="Ressource introuvable"):
        super().__init__(message,title,404 )


class ParamsException(ExceptionManager):
    def __init__(self, message="Les données soumises sont invalides ou incomplètes."):
        super().__init__(message ,"Paramètres incorrects",400)


class TooManyRequests(ExceptionManager):
    def __init__(self, message="Trop de requêtes ont été envoyées en peu de temps. Merci de patienter avant de réessayer."):
        super().__init__(message, "Requêtes trop fréquentes",429)


_all_ = ['CryptoManager']
from argon2.exceptions import VerifyMismatchError

import base64
import json
import os
import secrets
import time
from base64 import b64encode, b64decode

import base58
from argon2 import PasswordHasher
from cryptography.fernet import Fernet
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import x448
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives.kdf.hkdf import HKDF

from . import toolbox


class CryptoManager:
    """
    Gère les opérations cryptographiques (génération de clés, chiffrement/déchiffrement, stockage en base).

    - Utilise X448 pour l'échange de clés.
    - AES-GCM pour le chiffrement symétrique.
    - Clés stockées chiffrées dans la base avec fingerprint.
    """
    master_key = None
    ph = PasswordHasher()

    @staticmethod
    def set_master_key(master_key: str | bytes):
        CryptoManager.master_key = toolbox.ensure_string(master_key)

    @staticmethod
    def __generate_fingerprint(public_bytes: bytes) -> str:
        """Génère un fingerprint SHA-256 pour une clé publique"""
        public_bytes = toolbox.ensure_bytes(public_bytes)  # on s'assure que c'est du bytes
        fingerprint = hashes.Hash(hashes.SHA256())
        fingerprint.update(public_bytes)
        fingerprint_result = fingerprint.finalize()
        return base58.b58encode(fingerprint_result).decode()

    @staticmethod
    def get_fingerprint(public_key_bytes) -> str:
        """Retourne un digest SHA256 tronqué (16 premiers caractères) d'une clé publique."""
        public_key = x448.X448PublicKey.from_public_bytes(public_key_bytes)

        raw_bytes = public_key.public_bytes(encoding=serialization.Encoding.Raw, format=serialization.PublicFormat.Raw)
        digest = hashes.Hash(hashes.SHA256(), backend=default_backend())
        digest.update(raw_bytes)
        full_digest = digest.finalize()
        base58_digest = base58.b58encode(full_digest).decode()
        return base58_digest[:16]

    @staticmethod
    def verify_fingerprint(public_bytes: str | bytes, stored_fingerprint: str) -> bool:
        """Vérifie que l'empreinte générée correspond à celle stockée."""
        fp1 = CryptoManager.__generate_fingerprint(public_bytes)
        return fp1 == stored_fingerprint

    @staticmethod
    def generate_keys() -> tuple[bytes, bytes, str]:
        """Génère une paire de clés X448, stocke en DB et retourne les éléments."""
        private_key = x448.X448PrivateKey.generate()
        private_bytes = private_key.private_bytes(encoding=serialization.Encoding.Raw,
                                                  format=serialization.PrivateFormat.Raw,
                                                  encryption_algorithm=serialization.NoEncryption())
        public_key = private_key.public_key()
        public_bytes = public_key.public_bytes(encoding=serialization.Encoding.Raw,
                                               format=serialization.PublicFormat.Raw)
        finger_print = CryptoManager.__generate_fingerprint(public_bytes)

        return public_bytes, private_bytes, finger_print

    @staticmethod
    def exchange_shared_key(private_key_bytes: bytes, public_key_bytes: bytes):
        """Échange de clés X448 pour générer une clé partagée"""
        # public_key_bytes = public_key_bytes

        private_key = x448.X448PrivateKey.from_private_bytes(private_key_bytes)
        public_key = x448.X448PublicKey.from_public_bytes(public_key_bytes)
        shared_key = private_key.exchange(public_key)

        return shared_key

    @staticmethod
    def encrypt_data(dc: str | bytes, shared_key, client_secret) -> str:
        # Générer les clés X448 pour l'expéditeur et le récepteur
        iv = os.urandom(12)  # IV pour AES-GCM
        aes_key = CryptoManager.derive_aes_key(shared_key,
                                               client_secret)  # Dériver la clé AES à partir de la clé partagée
        encryptor = Cipher(algorithms.AES(aes_key), modes.GCM(iv), backend=default_backend()).encryptor()
        data_bytes = toolbox.ensure_bytes(dc)
        ciphertext = encryptor.update(data_bytes) + encryptor.finalize()
        tag = encryptor.tag

        encryted_data = {'ciphertext': CryptoManager.encode_secret(ciphertext), 'iv': CryptoManager.encode_secret(iv),
                         'tag': CryptoManager.encode_secret(tag)}
        return json.dumps(encryted_data)

    @staticmethod
    def encrypt_message(dc: dict | str, receiver_public_key, sender_private_key, piment) -> str:
        # Générer les clés X448 pour l'expéditeur et le récepteur
        if isinstance(dc, dict):
            dc = json.dumps(dc)

        shared_key = CryptoManager.exchange_shared_key(sender_private_key, receiver_public_key)
        # Chiffrer le message

        return CryptoManager.encrypt_data(dc, shared_key, piment)

    @staticmethod
    def encrypt_password(password):
        """
        Argon2 est une fonction de hachage de mot de passe lauréate du concours Password Hashing Competition.
        """
        return CryptoManager.ph.hash(password)

    @staticmethod
    def decrypt_encrypted_data(encrypted_data: str, shared_key: str | bytes, secret_key: str | bytes) -> str:
        """Déchiffre les données avec AES-GCM en utilisant la clé partagée dérivée"""
        dc = json.loads(encrypted_data)

        iv = CryptoManager.decode_secret(dc['iv'])
        tag = CryptoManager.decode_secret(dc['tag'])
        ciphertext = CryptoManager.decode_secret(dc['ciphertext'])

        aes_key = CryptoManager.derive_aes_key(shared_key, secret_key)  # Dériver la clé AES à partir de la clé partagée
        cipher_decrypt = Cipher(algorithms.AES(aes_key), modes.GCM(iv, tag), backend=default_backend()).decryptor()
        decypted_message = cipher_decrypt.update(ciphertext) + cipher_decrypt.finalize()

        return toolbox.ensure_string(decypted_message)

    @staticmethod
    def decrypt_encrypted_message(message_crypted, receiver_private_key: bytes, sender_public_key: bytes,
                                  salt_ket: bytes) -> str:
        """Déchiffre un message JSON en utilisant AES-GCM et l'échange de clés X448"""
        shared_key = CryptoManager.exchange_shared_key(receiver_private_key, sender_public_key)
        return CryptoManager.decrypt_encrypted_data(message_crypted, shared_key, salt_ket)

    @staticmethod
    def check_password(password, hasher):
        """
        Vérification du mot de passe
        """
        try:
            return CryptoManager.ph.verify(hasher, password)
        except VerifyMismatchError:
            return False

    @staticmethod
    def generate_bytes_secret(x: int = 24) -> bytes:
        """
        Génère une clé secrète aléatoire sous forme d'octets sécurisés.

        :param x: La longueur de la clé en octets. Par défaut, la longueur est de 24 octets.
        :type x: Int
        :return: Une séquence d'octets aléatoires sécurisés.
        :rtype: Bytes

        : Example :

        >>> CryptoManager.generate_bytes_secret(16)
        b'\x8a\xe4\xb3\xf0\xd5\xc7\xe9\x89\xaa\xa3~\xd1\x92\xc5\xb5'
        """
        return secrets.token_bytes(x)

    @staticmethod
    def generate_hex_secret(x=16) -> str:
        """Génère une clé secrète aléatoire sous forme de chaîne hexadécimale.

        :param x: La longueur de la chaîne hexadécimale. Par défaut, la longueur est déterminée automatiquement.
        :type x: Int, optional
        :return: Une chaîne hexadécimale aléatoire.
        :rtype: Str

        >>> CryptoManager.generate_hex_secret(16)
        'a1b2c3d4e5f6'
        """
        return secrets.token_hex(x)

    @staticmethod
    def code_unique(texte):
        timestamp = str(int(time.time()))
        texte_timestamp = texte + timestamp
        hasher = hashes.Hash(hashes.SHA1())
        hasher.update(texte_timestamp.encode())
        code_hash = hasher.finalize().hex()
        return code_hash[:8]  # Récupère les 8 premiers caractères du code de hachage

    @staticmethod
    def derive_aes_key(auth_key: str | bytes, salt_key: str | bytes) -> bytes:
        """Dérive une clé AES de 256 bits à partir de la clé partagée X448 en utilisant HKDF"""
        # Utiliser HKDF pour dériver une clé AES 256 bits
        auth_key = toolbox.ensure_bytes(auth_key)
        salt_key = toolbox.ensure_bytes(salt_key)

        hkdf = HKDF(algorithm=hashes.SHA256(), length=32,  # 32 bytes = 256 bits
                    salt=salt_key, info=b"SharedKeyDerivation", backend=default_backend())
        return hkdf.derive(auth_key)

    @staticmethod
    def fernet_engine(auth_key: str | bytes, salt_key: str | bytes) -> Fernet:
        """
            Déchiffre les données d'un fichier et les retourne.

            Args:
            fs (str): Le chemin du fichier contenant les données chiffrées.

            Returns:
            str: Les données déchiffrées.
        """
        key_derived = CryptoManager.derive_aes_key(auth_key, salt_key)
        fernet_key = base64.urlsafe_b64encode(key_derived)
        return Fernet(fernet_key)

    @staticmethod
    def basic_auth_encode(s_id: str, s_sekret: str):
        """Generation d'un autorization basic de type ID:SECRET"""
        return CryptoManager.encode_secret(f"{s_id}:{s_sekret}")

    @staticmethod
    def basic_auth_token(s_id: str, s_sekret: str):
        """Generation d'un autorization basic de type ID:SECRET"""
        access_token = CryptoManager.basic_auth_encode(s_id, s_sekret)
        return f"Basic {access_token}"

    @staticmethod
    def basic_auth_decode(access_token):
        """Generation d'un autorization basic de type ID:SECRET"""
        access_token = toolbox.ensure_string(CryptoManager.decode_secret(access_token))
        return access_token.split(":")

    @staticmethod
    def encode_secret(secret_data: bytes | str) -> str:
        secret_data = toolbox.ensure_bytes(secret_data)
        encrypted_data = b64encode(secret_data)
        return toolbox.ensure_string(encrypted_data)

    @staticmethod
    def decode_secret(secret_data: bytes | str) -> bytes:
        secret_data = toolbox.ensure_bytes(secret_data)
        encrypted_data = b64decode(secret_data)
        return encrypted_data

    @staticmethod
    def encrypt_secret(sct, auth_key, salt_key) -> str:
        engine = CryptoManager.fernet_engine(auth_key, salt_key)
        secret_data = engine.encrypt(sct)

        return toolbox.ensure_string(secret_data)

    @staticmethod
    def decrypt_secret(encrypted_secret: bytes | str, auth_key: bytes | str, salt_key: bytes | str) -> bytes:
        engine = CryptoManager.fernet_engine(auth_key, salt_key)
        t = base64.b64encode(toolbox.ensure_bytes(encrypted_secret))

        return engine.decrypt(encrypted_secret)

    @staticmethod
    def encrypt_file(js: dict, fs: str, auth_key: str, salt_key: str):
        """
            Déchiffre les données d'un fichier et les retourne.

            Args:
            fs (str): Le chemin du fichier contenant les données chiffrées.

            Returns:
            str: Les données déchiffrées.
        """
        document = toolbox.ensure_bytes(json.dumps(js))

        sensitive_data = toolbox.ensure_bytes(CryptoManager.encrypt_secret(document, auth_key, salt_key))

        with open(fs, 'wb') as file:
            file.write(sensitive_data)

    @staticmethod
    def decrypt_file(fs: str, auth_key: str, salt_key: str):
        """
            Déchiffre les données d'un fichier et les retourne.

            Args:
            fs (str): Le chemin du fichier contenant les données chiffrées.

            Returns:
            str: Les données déchiffrées.
        """
        try:
            with open(fs, 'rb') as file:
                encrypted_data = file.read()

            sensitive_data = CryptoManager.decrypt_secret(encrypted_data, auth_key, salt_key)
            return json.loads(sensitive_data)
        except Exception as ex:
            raise Exception('Erreur clé de chiffrage : {}'.format(ex))



import pytest

from src.dreamtools import file_manager
from src.dreamtools import date_manager
from src.dreamtools.mailing_manager import MailController

from tests import Constantine

reference = 'CWI-REQ-20250716-001'
date_test = date_manager.get_today_str(fm='%d-%m-%Y')

#modifier les parametre
dest_name='Jane DOE'
dest_mail='jane.doe@emailk.com'
smtp_url = 'smtp.domaine.net'
smtp_host = 465
smtp_mail = 'ne-pas-repondre@comaine.net'
smtp_password = 'mot-de-passe'
bot_name="Bot Intelligent"

# 🔧 Fixture de configuration mail (dépend de fixation)
@pytest.fixture(scope="session")
def mail_fix(fixation):
    path_template = file_manager.path_build(Constantine.APP_DIR, 'config/mailing.yml')  #emplacement du fichier templates
    Constantine.mailer =  MailController (smtp_url, smtp_host, smtp_mail, smtp_password, path_template, SMTP_USER_NAME=bot_name)

def test_sending_preconfig(mail_fix):
    result = Constantine.mailer.presend(dest_mail,'mail_test', date_test=date_test, dest_name=dest_name, reference=reference)
    assert result == True

def test_sending_custom(mail_fix):
    mail = {
        'subject': 'Test subject',
        'text': """ Bonjour Bonjour {dest_name},
        Nous sommes le {date_test}
        
        Lorem ipsum dolor sit amet, consectetur adipiscing elit. Praesent vitae tincidunt dui, porttitor eleifend dui. Duis a porta neque. Suspendisse ut tincidunt justo. Vivamus ut ipsum eu nunc feugiat sodales. Nulla id ligula semper, tincidunt dolor quis, ullamcorper sapien. Aliquam massa felis, placerat id elit sit amet, viverra malesuada velit. Nullam fringilla, urna a mattis semper, justo massa dapibus tellus, at ornare sapien eros et nulla. Etiam venenatis orci pretium, elementum mi nec, bibendum erat. In eget sagittis sapien. Etiam et libero odio. Donec facilisis dapibus arcu, vel aliquet lectus eleifend ac.
Vestibulum congue, tortor sed consequat ultricies, ex felis elementum nibh, sit amet sodales sapien massa nec nunc. Phasellus dapibus malesuada justo et tincidunt. Nulla facilisi. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Pellentesque consectetur eleifend diam sed ullamcorper. Donec scelerisque scelerisque libero sit amet laoreet. Suspendisse accumsan ut urna at dapibus. Aliquam finibus tellus quis finibus semper. Pellentesque sit amet efficitur massa. Suspendisse eu lectus nibh. Nunc eget pretium tellus, id maximus velit.

Mauris finibus sagittis dui, ut sodales justo dapibus sed. Donec tincidunt posuere mollis. Phasellus rhoncus venenatis massa, eget vehicula neque dignissim eu. Sed pretium vel est quis convallis. Duis a est odio. Mauris et sem eu neque euismod malesuada a in libero. Praesent luctus fringilla nisi, sit amet ultricies elit varius vitae. Nunc eget tortor at velit maximus volutpat non in nisl. In aliquet, dui vel lacinia auctor, leo est interdum metus, quis mollis diam odio a leo. Sed at semper turpis. Quisque congue ligula nec eros hendrerit luctus. Pellentesque a massa nibh. Nullam elit elit, condimentum vel sapien quis, pharetra porttitor elit. Mauris sollicitudin suscipit rutrum. Etiam tempor ex id laoreet rutrum. """,
        'html': """ <b>Bonjour Bonjour {dest_name},</b> 
        Nous sommes le {date_test}

        <p>Lorem ipsum dolor sit amet, consectetur adipiscing elit. Praesent vitae tincidunt dui, porttitor eleifend dui. Duis a porta neque. Suspendisse ut tincidunt justo. Vivamus ut ipsum eu nunc feugiat sodales. Nulla id ligula semper, tincidunt dolor quis, ullamcorper sapien. Aliquam massa felis, placerat id elit sit amet, viverra malesuada velit. Nullam fringilla, urna a mattis semper, justo massa dapibus tellus, at ornare sapien eros et nulla. Etiam venenatis orci pretium, elementum mi nec, bibendum erat. In eget sagittis sapien. Etiam et libero odio. Donec facilisis dapibus arcu, vel aliquet lectus eleifend ac.

Vestibulum congue, tortor sed consequat ultricies, ex felis elementum nibh, sit amet sodales sapien massa nec nunc. Phasellus dapibus malesuada justo et tincidunt. Nulla facilisi. Lorem ipsum dolor sit amet, consectetur adipiscing elit. Pellentesque consectetur eleifend diam sed ullamcorper. Donec scelerisque scelerisque libero sit amet laoreet. Suspendisse accumsan ut urna at dapibus. Aliquam finibus tellus quis finibus semper. Pellentesque sit amet efficitur massa. Suspendisse eu lectus nibh. Nunc eget pretium tellus, id maximus velit.

Mauris finibus sagittis dui, ut sodales justo dapibus sed. Donec tincidunt posuere mollis. Phasellus rhoncus venenatis massa, eget vehicula neque dignissim eu. Sed pretium vel est quis convallis. Duis a est odio. Mauris et sem eu neque euismod malesuada a in libero. Praesent luctus fringilla nisi, sit amet ultricies elit varius vitae. Nunc eget tortor at velit maximus volutpat non in nisl. In aliquet, dui vel lacinia auctor, leo est interdum metus, quis mollis diam odio a leo. Sed at semper turpis. Quisque congue ligula nec eros hendrerit luctus. Pellentesque a massa nibh. Nullam elit elit, condimentum vel sapien quis, pharetra porttitor elit. Mauris sollicitudin suscipit rutrum. Etiam tempor ex id laoreet rutrum.</p>"""}

    result = Constantine.mailer.presend(dest_mail, 'custom', date_test=date_test, dest_name=dest_name, template_email=mail)
    assert result == True



# -*- coding: utf-8 -*-
# project/dreamtools-dreamgeeker/image_manager.py
_all_ = ['ImageManager', 'TYPE_IMG_JPEG', 'TYPE_IMG_PNG', 'TYPE_IMG_GIF', 'TYPE_IMG_WEBP', 'TYPE_IMG_SVG']

import copy
import os
from io import BytesIO

from PIL import Image, ImageFile, ImageDraw
from PIL.ExifTags import TAGS
from PIL.TiffImagePlugin import ImageFileDirectory_v2

from . import file_manager
from .controller_manager import ControllerEngine
from .tracking_manager import TrackingManager

TYPE_IMG_JPEG = 'JPEG'
TYPE_IMG_PNG = 'PNG'
TYPE_IMG_GIF = 'GIF'
TYPE_IMG_WEBP = 'WEBP'
TYPE_IMG_SVG = "SVG"

"""
Class CImagine
=============================

Class permettant le traitement d'une image png convertit en jpg avec prise en charge de la transparence (fond blanc)


pathfile : dreamtools-dreamgeeker/pyimaging

"""

MAX_SIZE = 640
MIN_SIZE = 200


class ImageManager(object):
    size_max: int
    size_thumb_max: int

    def __init__(self, src, dest, size_max=MAX_SIZE, size_thumb_max=MIN_SIZE):
        """
        Preparation image  pour traitement
        ==================================
        Les images sont convertit au format JPEG

        :param int size_max: taille maximum d'une image
        :param int size_thumb_max: taille miniature et minimum

        """
        self.extension = file_manager.file_extension(src).upper()

        if self.extension == "JPG":
            self.extension = TYPE_IMG_JPEG

        if self.extension not in [TYPE_IMG_JPEG, TYPE_IMG_PNG, TYPE_IMG_GIF, TYPE_IMG_WEBP]:
            raise ValueError("Format image non supporté")

        self.size_max = size_max
        self.size_thumb_max = size_thumb_max

        self.img = Image.open(src)
        self.exif = None

        self._size = self.img.size
        self._format = self.img.format.lower()

        self.file = file_manager.file_extension_less(dest)  # on s'assure de retirer l'extension

        self.resize()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        try:
            self.img.close()
        except:
            pass

    @property
    def _size(self):
        return self.w, self.h

    @_size.setter
    def _size(self, s):
        self.w, self.h = s

    def image_to_rgb(self):
        if self.img.mode != 'RGB':
            self.img = self.img.convert('RGB')
            self.extension = TYPE_IMG_JPEG

    def white_background(self):
        """
        Retourne l'image avec un fond blanc appliqué si l'image comporte de la transparence.
        :return: PIL.Image avec fond blanc
        """

        # Si l'image a un canal alpha (RGBA), on l’utilise comme masque
        if self.img.mode in ("RGBA", "LA"):
            bg = Image.new("RGB", self._size, (255, 255, 255))
            bg.paste(self.img, (0, 0), self.img)
            return bg
        return self.img

    def redraw_border(self, shape="rect", wc=False):
        """
            fs = '/home/dreamgeeker/Images/2151910934.jpg'
            fs_out = '/home/dreamgeeker/Images/tests.jpg'

            imager = ImageManager(fs, fs_out.format('circ'))
            imager.img = imager.redraw_border('circ')
            imager.save('png')
            imager = ImageManager(fs, fs_out.format('circ_border'))
            imager.img = imager.redraw_border('circ', True)
            imager.save('png')"""

        padding = 5 if wc else 0
        diameter = 2 * padding
        box = (self.img.width + diameter, self.img.height + diameter)

        if shape == "rect":
            bg = Image.new("RGB", box, (255, 255, 255))
            bg.paste(self.white_background(), (padding, padding))
        else:  # shape == "circ":
            self.img.convert("RGBA")
            bg = Image.new("RGBA", box, (0, 0, 0, 0))
            if wc:
                draw = ImageDraw.Draw(bg)
                draw.ellipse((0, 0, box[0], box[1]), fill=(255, 255, 255, 255))

            # Masque circulaire
            mask = Image.new("L", self.img.size, 0)
            draw = ImageDraw.Draw(mask)
            draw.ellipse((0, 0, self.img.width, self.img.height), fill=255)

            pos_x = padding
            pos_y = padding

            # Coller l'image sur le cercle blanc au centre
            bg.paste(self.img, (pos_x, pos_y), mask)

        return bg

    def resize(self):
        """ Redimensionnement de l'image au format jpg


        :return:
        """
        if self.h < self.size_thumb_max or self.w < self.size_thumb_max:
            raise Exception("Image trop petite")
        elif self.h < self.size_max and self.w < self.size_max:
            return

        if self.w >= self.h:
            coeff = self.w / self.size_max
        else:
            coeff = self.h / self.size_max

        h = int(self.h / coeff)
        w = int(self.w / coeff)


        self.img = self.img.resize((w, h))

    def recadre(self, w, h):
        """"""
        for coeff in range(11):
            if w % coeff == 0 and h % coeff == 0:
                w //= coeff
                h //= coeff
                w, h = self.recadre(w, h)
                break

        return w, h

    def rationalize(self, r_w: int, r_h: int):
        """ Redimensionnement de l'image au format jpg

        :return:
        """
        w = self.w
        h = w * r_h // r_w

        self.img = self.img.resize((w, h))

    def save(self, frm: str | None = None):
        """ Thumb Image
        :param frm:
        """

        frm = frm.lower() if frm else self._format
        file_name = self.file + '.' + frm
        frm = frm.upper()

        # certaines conversions nécessitent convert()
        if frm == 'JPEG' and self.img.mode in ('RGBA', 'LA'):
            # JPEG ne supporte pas alpha → convertir en RGB
            background = Image.new("RGB", self.img.size, (255, 255, 255))
            background.paste(self.img, mask=self.img.split()[-1])  # utiliser l'alpha comme masque
            self.img = background

        file_manager.makedirs(file_manager.parent_directory(file_name))

        if frm == TYPE_IMG_PNG and self.exif:
            self.img.save(file_name, exif=self.exif, quality=90, optimize=True)
        else:
            self.img.save(file_name, format=frm, quality=90, optimize=True)

    def generate_thumb(self):
        """ Thumb Image       """
        thumb = copy.deepcopy(self)
        thumb.img.thumbnail((self.size_thumb_max, self.size_thumb_max))
        thumb.file += "_thumb"
        return thumb

    def save_thumb_image(self, frm: str = TYPE_IMG_JPEG):
        """ Thumb Image
        """
        thumb = self.generate_thumb()
        thumb.save(frm=frm)

    def to_badge(self):
        """Redimensionnement de l'image

        :return:
        """
        thumb = copy.deepcopy(self)
        thumb.img = thumb.redraw_border('circ', wc=True)
        thumb.img.thumbnail((64, 64))
        thumb.file += "_ico"

        return thumb

    def save_image_jpeg(self):
        self.image_to_rgb()
        self.save(frm=TYPE_IMG_JPEG)

    def protected(self, artist, description):
        """Ajoute un nom d'artist et le copyright d'une image"""

        _TAGS_r = dict(((v, k) for k, v in TAGS.items()))

        # Image File Directory
        ifd = ImageFileDirectory_v2()
        ifd[_TAGS_r["Artist"]] = artist
        ifd[_TAGS_r["Copyright"]] = 'Tous droits réservés'
        ifd[_TAGS_r["Description"]] = description

        out = BytesIO()
        ifd.save(out)

        self.exif = b"Exif\x00\x00" + out.getvalue()
        self.img.save(self.file, TYPE_IMG_JPEG, exif=self.exif)

    @staticmethod
    def directory_parsing(main_directory: str):
        """
        Redimensionne toutes les images contenu dans un répertoire donné + thumb
        :param main_directory:
        """
        for f in os.listdir(main_directory):
            f_path = os.path.join(main_directory, f)
            if os.path.isfile(f_path):
                imager = ImageManager(f_path, f_path)
                imager.save()
                imager.save_thumb_image()

    @staticmethod
    def treat_uploading(fs, fp):
        """
        Enregistrement d'une image uploaded (byte)

        :param file fs: filestream from flask request
        :param str fp: filepath d'enregistrement
        :return:
        """
        import uuid

        tmp = uuid.uuid1().int >> 64
        tmp_image = file_manager.path_build(ControllerEngine.TMP_DIR, str(tmp))

        try:
            TrackingManager.flag(f'[imgrecorder] Image temp recording : {tmp}')
            fs.save(tmp_image)

            TrackingManager.flag(f'[imgrecorder] Image treatment: {tmp} | {fp}')
            o = ImageManager(tmp_image, fp)
            o.save()

            TrackingManager.flag('[imgrecorder] Image thumbing')
            o.save_thumb_image()

            file_manager.remove_file(tmp_image)

            return True
        except Exception as e:
            TrackingManager.exception_tracking(e, '[imgrecorder] Aïe')
            return False

    @staticmethod
    def get_file_size(i_p):
        """

        :param i_p: image_path
        :return:
        """
        size_bytes = None
        try:
            # Obtenir la taille du fichier image en octets
            size_bytes = os.path.getsize(i_p)
        finally:
            return size_bytes  # Retourne None si la taille ne peut pas être obtenue



# -*- coding: utf-8 -*-

"""
Module de fonctions basiques
=============================

Liste de fonctions utiles

pathfile : dreamtools-dreamgeeker/tools

Constantes globales
--------------------
.. note::

.. warning::
    Il faut configurer l'application afin d'avoir accès au variable PROJECT_DIR, APP_NAME, TMP_DIR

"""
import ast
import base64
import random
import re
import sys
from random import choice, randint
from string import punctuation, ascii_letters, digits
from urllib.parse import urlparse

import dns.resolver
import httpx
from bs4 import BeautifulSoup

RGX_ACCENTS = 'àâäãéèêëîïìôöòõùüûÿñç'
RGX_PUNCT = '#!?$%&_@*+-'
RGX_EMAIL = r'^[a-z0-9_.+-]+@[a-z0-9-]+\.[a-z0-9-.]+$'
RGX_PWD = r'^(?=(?:.*[a-z]){2,})(?=(?:.*[A-Z]){2,})(?=(?:.*\d){2,})(?=(?:.*[!@#$%^&*()_+\-=\[\]{};:\'",.<>\/?\\|`~]){2,}).{9,32}$'
RGX_PHONE = r'^(?:0\d(?:[ .-]?\d{2}){4}|(?:\+|00)\d{1,3}(?:[ .-]?\d+){4,})$'
RGX_URL = r'https?:\/\/(www\.)?[-a-z0-9@:%._\+~#=]{1,256}\.[a-z0-9()]{1,6}\b([-a-z0-9()@:%_\+.~#?&//=]*)'

RGX_PHONE_PATTERN = re.compile(RGX_PHONE)
RGX_EMAIL_PATTERN = re.compile(RGX_EMAIL)
RGX_PWD_PATTERN = re.compile(RGX_PWD)
RGX_URL_PATTERN = re.compile(RGX_URL)

import pyperclipfix
import json

def clipboard_copy():
    pyperclipfix.copy('The text to be copied to the clipboard.')


def clipboard_paste():
    pyperclipfix.paste()


'The text to be copied to the clipboard.'


def print_err(*args, **kwargs):
    """Ecriture sur le flux erreur de la console

    :param args: arguments 1
    :param kwargs: arguemnts2
    """
    print(*args, file=sys.stderr, **kwargs)


def get_string(v):
    """ Convertion d'une valeur en chaine

    :param v: valeur à convertir
    :rtype: str, None en cas d'erreur

    """
    return str(v) if v else ''


def clean_space(ch):
    """ Nettoyage des espaces "superflus"

    * Espaces à gouche et à droite supprimés
    * Répétition d'espace réduit

    :Exemple:
        >>> chaine = 'Se  réveiller au matin        de sa destiné    ! !           '
        >>> clean_space (chaine)
        'Se réveiller au matin  de sa destiné ! !'

    """
    s = get_string(ch)
    return re.sub(r'\s{2,}', ' ', s.strip())


def clean_allspace(ch, very_all=True):
    """Nettoyage de tous les espace et carateres vides

    :param str ch: Chaine à nettoyer
    :param bool very_all: caractère vide aussi, True (False = Espaces uniquement)

    :Exemple:
        >>> chaine = 'Se  réveiller au matin        de sa destiné !'
        >>> clean_allspace (chaine)
        'Seréveilleraumatindesadestiné!'

    """
    c = r'\s' if very_all else '[ ]'
    s = get_string(ch)

    return re.sub(c, '', s.strip())


def clean_coma(ch, w_punk=False):
    """ Supprime les accents/caractères spéciaux du texte source en respectant la casse

    :param ch: Chaine de caractere à "nettoyer"
    :param w_punk: indique si la punctuation est à nettoyer ou pas (suppression)

    :Exemple:
        >>> s = "Se  réveiller au matin    de sa destiné, l'ideal soi-meme !!"
        >>> clean_coma (s)
        'Se seveiller au matin (ou pas) de sa destine, l'ideal soi-meme !!''
        >>> clean_coma (s, True)
        'Se reveiller au matin ou pas de sa destine l ideal soi meme'

    """
    if w_punk:
        # Nettoyage caractere spéciaux (espace...)
        ch = re.sub("""[-_']""", ' ', ch)
        o_rules = str.maketrans(RGX_ACCENTS, 'aaaaeeeeiiioooouuuync', punctuation)
    else:
        o_rules = str.maketrans(RGX_ACCENTS, 'aaaaeeeeiiioooouuuync')

    return clean_space(ch.translate(o_rules).swapcase().translate(o_rules).swapcase())


def clean_master(ch):
    """ Supprime les accents, caractères spéciaux et espace du texte source

    :param str ch: Chaine de caractere à "nettoyer"
    :return str: chaine sans accents:car. spéciaux ni espace en minuscule

    :Exemple:
        >>> s = 'Se  réveiller au matin  (ou pas) de sa destiné !'
        >>> clean_master (s)
        'sereveilleraumatinoupasdesadestine

    """
    return clean_allspace(clean_coma(ch, True)).lower()


def int_to_hexa(v):
    """ Conversion d'une valeur en hexadécimal

    :param int v: nombre à convertir
    :returns: valeur en hexadécimal
    :rtype: str

    """
    return hex(int(v))


def add_hexa(h, v):
    """Additionne une valeur hexadécimal

    :param str h: valeur hexadécimal
    :param int v: valeur entière à ajouter
    :return: valeur additionné en hexedécimal

    :Example:
        >>> hx = '0x129'
        >>> add_hexa(hx, 2)
        0x12b

    """

    v += int(h, 16)
    return hex(v)


def compare_hex(hx_a, hx_b):
    """Compare deux valeurs  hexadécimales

    :param str hx_a:
    :param str hx_b:

    :return int:
        * 0 : hx_a == hx_b
        * 1 : hx_a > hx_b
        * -1 : hx_a < hx_b
    """
    v = int(hx_a, 16) - int(hx_b, 16)

    if v == 0:
        return 0
    else:
        return -1 if v < 0 else 1


def plain_hex(hx, s=3):
    """ Complète un chiffre hexadecimal en préfixant une valeur de zéro

    :param str hx: valeur hexadécimal
    :param int s: longeur chaine attendu
    :rtype: str:

    :Examples:
        >>> hxi = '0x129'
        >>> plain_hex(hxi, 5)
        0x00129

    """
    return hx[:2] + plain_zero(hx[2:], s)


def plain_zero(v, s):
    """Complete une valeur chaine de zéro

    :param v: valeur à completer
    :param s: taille chaine attendu préfixé de zerom

    :Exemple:
        >>> d = 5
        >>> plain_zero(d,3)
        '005'

    """

    s = '{:0>' + str(s) + '}'
    return s.format(v)


def check_password(s):
    """ Vérifie que la syntaxe d'une chaine répond au critère d'un mot de passe

    : Conditions :
        * Une majuscule
        * Une minuscule
        * Un chiffre
        * Un caractère spécial (@#!?$&-_ autorisé)

    :param str s: chaine à vérifier
    :return bool: True si la chaine est valide
    """
    return RGX_PWD_PATTERN.match(s)


def pwd_maker(i_size=12):
    """ Génération d'un password respectant les regles de password

    : Conditions :
        * deux majuscules
        * deux minuscules
        * deux chiffres
        * deux caractères spé&ciaux (@#!?$&-_ autorisés)

    :param int i_size: Nombre de caractères de la chaine
    :return: Mot de passe
    """

    l_t = list(ascii_letters + digits + RGX_PUNCT)
    password = ''
    s_chaine = ''

    while not check_password(password):
        s = choice(l_t)
        s_chaine += s
        password = s_chaine[-i_size:]
    return s_chaine


def code_maker(n=None, w_punk= False):
    if n is None:
        n = random.randint(6, 24)
    alphabet = ascii_letters + digits
    if w_punk:
        alphabet += RGX_PUNCT

    return ''.join(choice(alphabet) for _ in range(n))


def random_number(end, s=1):
    """ Génération d'un nombre aléatoire entre [1-end]  end caractère

    :param int end: valeur maximale (paut indiquer la taille si s=1)
    :param s: valeur de départ, default to 1
    :return: Un chiffre aléatoire

    : Exemple :
        >>> random_number (5)
        1 : Renvoie un chiffre entre 1 et 5
        >>> random_number (5,3)
        1 : Renvoie un chiffre entre 3 et 5
        >>> 4

    """
    return randint(s, end)


def append_to_list(v, ll: list):
    """ Ajout d'un item dans une liste avec gestion des doublons

    :param str v: valeur à ajouter
    :param list ll: liste

    """
    if v not in ll:
        ll.append(v)


def dictlist(k, v, d: dict):
    """ Ajout d'une valeur dans une liste d'un dictionnaire

    :param str k: clé dictionnaire
    :param v: valeur à ajouter
    :param dict[str, list[]] d: dictionnaire

    : Exemple :
        >>> dictionnaire= {}
        >>> dictlist('printemps', 'mar', dictionnaire)
        dictionnaire{'printemps', ['mars']}
        >>> dictlist('printemps', 'avril', dictionnaire)
        dictionnaire{'printemps', ['mars', ''avril']}
        >>> dictlist('printemps', 'mars', dictionnaire)
        dictionnaire{'printemps', ['mars', ''avril']}

    """
    if k is None or v is None: return

    if k not in d:
        d[k] = [v]
    elif v not in d[k]:
        d[k].append(v)


def str_to_dic(chaine: str) -> dict:
    """Convertion d'une chaine en dictionnaire

    :param str chaine:
    :rtype: dc

    :Exemple:
        >>> s_dic = "{'key':value}"
        >>> str_to_dic(s_dic)
        {'key': 'value'}

    """
    return ast.literal_eval(chaine)


def pop_dic(l_ids: list, dc: dict):
    """ Suppression d'une liste d'éléments d'un dictionnaire

    :param list l_ids : liste de clé à supprimer
    :param dict dc: dictionaire à nettoyer

    """
    if dc:
        for s in l_ids:
            if s in dc: del dc[s]


def find_key(value, dc: dict):
    """Recherche une clé d'un dictionnaire à partir de sa valeur"""

    for k, v in dc.items():
        if v == value:
            return k
    return None


def html_to_text(html: str) -> str:
    """
    Extrait le texte brut d'un contenu HTML et nettoie les espaces.

    :param html: Chaîne HTML à traiter
    :return: texte brut nettoyé

    Exemple :
        >>> html_to_text('<p>Hello <b>world</b> !</p>')
        'Hello world !'
    """
    soup = BeautifulSoup(html, "html.parser")
    text = soup.get_text(strip=True)
    return clean_space(text)


def is_empty(item):
    return (item is None) or (len(item) == 0)


def find_email(s):
    return list(map(lambda tp: tp[0], RGX_EMAIL_PATTERN.findall(s)))


def has_mx_record(domain: str) -> bool:
    try:
        records = dns.resolver.resolve(domain, "MX")
        return len(records) > 0
    except (dns.resolver.NoAnswer, dns.resolver.NXDOMAIN, dns.resolver.NoNameservers, dns.exception.Timeout):
        return False


def is_valid_email(v: str) -> bool:
    try:
        v = clean_allspace(v).lower()
        if RGX_EMAIL_PATTERN.fullmatch(v):
            domain = v.split('@')[-1]
            return has_mx_record(domain)
    except Exception as ex:
        print(ex)
    return False


def is_valid_password(v: str) -> bool:
    v = clean_allspace(v)
    return bool(RGX_PWD_PATTERN.fullmatch(v))


def is_valid_phone(v: str) -> bool:
    v = clean_allspace(v)
    return bool(RGX_PHONE_PATTERN.fullmatch(v))


def is_valid_url(link: str) :
    # Force ajout du schéma si manquant
    try:
        if not link.startswith(('http://', 'https://')):
            link = f'https://{link}'

        url = clean_allspace(link).lower()
        if not RGX_URL_PATTERN.fullmatch(url):
            return False

        domain = urlparse(url).netloc
        if not domain:
            return False

        if is_valid_domain(domain):
            return verify_link(url)

    except Exception:
        return False


def is_valid_domain(domain: str) -> bool:
    try:
        dns.resolver.resolve(domain, "A")  # IPv4
        return True
    except dns.resolver.NXDOMAIN:
        return False
    except dns.resolver.NoAnswer:  # Essaye avec AAAA si pas de réponse IPv4
        try:
            dns.resolver.resolve(domain, "AAAA")  # IPv6
            return True
        except (dns.resolver.NXDOMAIN, dns.resolver.NoAnswer, dns.resolver.NoNameservers, dns.exception.Timeout):
            return False
    except (dns.resolver.NoNameservers, dns.exception.Timeout):
        return False


def verify_link(url: str, max_redirects: int = 3) -> str | bool:
    """
    Vérifie une URL HTTP/HTTPS et renvoie l'URL finale valide, ou False si invalide.

    :param url: URL à tester (ex : 'example.com')
    :param max_redirects: Nombre maximum de redirections autorisées
    :return: URL fonctionnelle (str) ou False
    """

    s = clean_allspace(url)

    try:
        with httpx.Client(follow_redirects=False, timeout=10.0) as client:
            for _ in range(max_redirects):
                response = client.head(s)
                if response.status_code in (301, 302) and "location" in response.headers:
                    s = response.headers["location"]
                    continue

                # Fallback GET si HEAD renvoie 405 ou comportement suspect
                if response.status_code in (405, 403, 400):
                    response = client.get(s)

                return s if response.is_success else False

            return False  # Trop de redirections
    except httpx.RequestError:
        return False


def split_by_crlf(s):
    return [v for v in s.splitlines() if v]


def init_and_set_dict(dc, key, value):
    if dc is None:
        dc = {}

    dc[key] = value


def dictionary(dictionnaire):
    document = None

    if type(dictionnaire) is dict:
        document = {k: dictionary(v) for k, v in dictionnaire.items() if v}
    elif type(dictionnaire) is list:
        document = [dictionary(v) for v in dictionnaire]
    elif dictionnaire is not None:
        try:
            document = dict(dictionnaire)
        except TypeError:
            document = dictionnaire

    return document


def ensure_bytes(element: str | bytes | bytearray) -> bytes:
    if isinstance(element, bytes):
        return element
    if isinstance(element, bytearray):
        return bytes(element)
    if isinstance(element, str):
        return element.encode("utf-8")  # ou "ascii" si tout est ASCII
    raise TypeError(f"Unsupported type: {type(element)}")


def ensure_string(element: str | bytes | bytearray) -> str:
    if isinstance(element, str):
        return element
    try:
        return element.decode("utf-8")
    except UnicodeDecodeError:
        return base64.b64encode(element).decode()

def ensure_dict (value) -> dict:
    if value:
        return value if isinstance(value, dict) else json.loads(value) if isinstance(value, str) else value
    return {}

def ensure_list (value) -> dict:
    if value:
        return value if isinstance(value, list) else json.loads(value) if isinstance(value, str) else value
    return {}



# -*- coding: utf-8 -*-
# features.py
import os
import shutil
import sys
from pathlib import Path


"""
Module complémentaire
============================
pathfile : dreamtools-dreamgeeker/features.py
"""

def lets_go():
    current_path = Path.cwd()
    config_dest = current_path / "configuration"
    config_source = Path(__file__).parent / ".config"

    os.makedirs(config_dest)
    shutil.copytree(config_source, config_dest)

    if config_dest.exists():
        print("⚠️  Le dossier `.config` existe déjà dans ce répertoire.")
        print("❗ Aucun fichier n’a été écrasé.")
        return

    try:
        shutil.copytree(config_source, config_dest)
        print("✅ Fichiers de configuration copiés dans le dossier `.config` du projet.")
        print("🛠️ Pensez à adapter les fichiers `log.yml` et `mailing.yml` à votre projet.")
    except Exception as e:
        print(f"❌ Erreur lors de la copie : {e}")
        sys.exit(1)